%%  --- INITIAL SETTINGS ---

% This work is licensed under the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 
% International License. To view a copy of this license, visit 
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to 
% Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.

%Start Flag: Put 1 if it is needed to load excel matrices and to build total times matrix.
%Put 0 otherwise.
FLAG_START = 1

%Solver Stop Flag: Put 1 if we want to exploit the first feasible solution
%found by the solver (generally through heuristics and not guaranteed to be optimal). Put 0 otherwise
FLAG_SOLVER_STOP = 0


time_start_overall = tic;

%Error Flag: set to 0 by default. If an error is detected in the checks introduced
%in the code, this flag will be set to 1 and the program stops.
ERROR_FLAG = 0;

%% FIRST STEP

if FLAG_START == 1

    clearvars -EXCEPT FLAG_START time_start_overall FLAG_ERROR FLAG_SOLVER_STOP
    close all
    clc

    %% --- Load Warehouse data from excel sheet ---

    % In this section, all the needed matrices are loaded from excel
    
    nodes_excel_matrix = xlsread('002_Validation.xlsx','A3:G164'); %nodes matrix [ ID  X  Y  onf  cat  horc  verc ]
    edges_excel_matrix = xlsread('002_Validation.xlsx','K3:M167'); %edges matrix [ ID  N1  N2 ]

    bay_identities_matrix = xlsread('002_Validation.xlsx','P3:R3'); %bay nodes matrix. [ ID  bn  bwf ]
    
    operation_nodes_matrix = xlsread('002_Validation.xlsx','W3:AD14'); %operation nodes matrix [ ID  ot  p  PROB.ABS  b  bt  brt  inf ]
    
    UPW_int_nodes_matrix = xlsread('002_Validation.xlsx','AI3:AR14'); %intermediate nodes matrix, for upward trip. By default, max 10 intermediate nodes can be accepted (can be increased if needed with minimal changes)
    RET_int_nodes_matrix = xlsread('002_Validation.xlsx','AW3:BF14'); %intermediate nodes matrix, for return trip. By default, max 10 intermediate nodes can be accepted (can be increased if needed with minimal changes)
    
    num_hor_corr = xlsread('002_Validation.xlsx','T12:T12'); %number of horizontal corridors
    num_ver_corr = xlsread('002_Validation.xlsx','T15:T15'); %number of vertical corridors

    %% --- Matrices Definition ---

    fprintf(' \n \n LOADING MATRICES FROM EXCEL ... \n')
    
    nodes = nodes_excel_matrix(:,1:3); %extract nodes matrix for Dijkstra algorithm, in form [ID X Y]

    edges = edges_excel_matrix(:,1:3); %extract segments matrix for Dijkstra algorithm, in form [ID N1 N2]
    
    num_nodes = size(nodes,1); %total number of nodes
    num_edges = size(edges,1); %total number of segments
    
    nodes_categories = nodes_excel_matrix(:,5); %vector of the nodes categories

    num_bays = size(bay_identities_matrix,1); %total number of bays
    
    num_operations = size(operation_nodes_matrix,1); %total number of operations
    
    corr_h = nodes_excel_matrix(:,6); %vector of horizontal corridors of each node
    
    corr_v = nodes_excel_matrix(:,7); %vector of vertical corridors of each node
    
    
    % Automatically set the Operation Node Flag of all the operation nodes
    % to 1.
    
    for j = 1:num_operations %scroll all operations
        
        nodes_excel_matrix(operation_nodes_matrix(j,1),4) = 1; %set op.node flag of the corresponding operation node to 1.
        
    end
    
    %% --- Parameters ---
    
    % Optimization Parameters
    
    cost_dynamic_WPT = 800; %cost [] of each WPT part.(4000 / 5).
    
    cost_static_WPT = 3000; %cost [] of a static WPT, to be eventually placed in bays.
    
    
    
    % Forklift data
    
    forklift_speed_avg = 6; % average forklift speed (vf) [km/h] ;

    node_spacing = 0.5; %node spacing (ns) used in the warehouse modelling, assumed the same for each couple of nodes [m]

    crossing_time = node_spacing / (forklift_speed_avg / 3.6 ); %crossing time (ct) [s]

    
    P_cons_max = 4300; %max power consumption of the forklift [W]

    P_cons_bay_op = P_cons_max * 0.628; %Power consumption when forklift is in bay operation phase
    P_cons_bay_rech = P_cons_max * 0.06; %Power consumption when forklift is in bay recharge phase
    P_cons_node_mov = P_cons_max * 0.558; %Power consumption when forklift is in node movement phase
    P_cons_node_op = P_cons_max * 0.628; %Power consumption when forklift is in node operation phase
    
    E_battery_max = 108*(10^6); %maximum energy storable in the battery [J]
    
    
    % WPT Technical Data
    
    Pn = 4000; %nominal power of WPT charging [W]
    
    static_efficiency = 0.90; % WPT system static efficiency
    dynamic_efficiency = 0.87; %WPT system dynamic efficiency, considering the forklift driver able to stay inside the 20% lateral tolerance

    
    
    % Logistic Data, Working Shift Duration, and Energetic customer
    % requirement
    
    ST_total = 8*3600; %duration of a turn [s]
    ST_breaks = 1*3600 + 15*60; %consider 1h 15min of breaks in a turn [s]
    ST_eff = ST_total - ST_breaks; %time of "operation" in a turn [s]
    
    k_breaks = 1; %parameter indicating the percentage of break time forklifts could be charged

    Delta_SoC_desired = 0; %Customer Requirement about the Delta_SoC_turn [%];
    
    %% --- Neighbors definition ---
    
    % In this section, all the required node neighbors are calculated,
    % through a set of functions.
    
    
    % calculate Radius-2 Horizontal Neighbors matrix
    
    fprintf(' \n \n SEARCHING FOR R2 HOR NODES NEIGHBORS ... \n')
    
    [Nodes_Neighbors_R2_Hor] = find_R2_horizontal_neighbors(nodes,edges,corr_h);
    
    
    % calculate Radius-2 Vertical Neighbors matrix
    
    fprintf(' \n \n SEARCHING FOR R2 VER NODES NEIGHBORS ... \n')
    
    [Nodes_Neighbors_R2_Ver] = find_R2_vertical_neighbors(nodes,edges,corr_v);
    
 
    % calculate Distance-5 Horizontal Neighbors matrix
    
    fprintf(' \n \n SEARCHING FOR D5 HOR NODES NEIGHBORS ... \n')
    
    [Nodes_Neighbors_D5_Hor] = find_D5_horizontal_neighbors(nodes,edges,corr_h);

    
    % calculate Distance-5 Vertical Neighbors matrix
    
    fprintf(' \n \n SEARCHING FOR D5 VER NODES NEIGHBORS ... \n')
    
    [Nodes_Neighbors_D5_Ver] = find_D5_vertical_neighbors(nodes,edges,corr_v);
    
    %% --- FIGURE 1 - Warehouse map ---

    % Draw a map of the warehouse with nodes and edges, for checking the
    % correctness of the set of data reported on excel sheet.

    figure(1)
    
    hold on

    plot(nodes(:,2),nodes(:,3),'b.') % plot the nodes
    for i=1:num_nodes %label the nodes
        if nodes_excel_matrix(i,4) == 1
            text(nodes(i,2),nodes(i,3)+0.13,...
                [' ' num2str(nodes(i,1))],'Color','r','FontWeight','b','FontSize',8)
        else
            text(nodes(i,2),nodes(i,3)+0.13,...
                [' ' num2str(nodes(i,1))],'Color','b','FontWeight','b','FontSize',8)
        end
        
        for j=1:num_bays %draw and label bay nodes
            
            if bay_identities_matrix(j,1) == i
                text(nodes(i,2),nodes(i,3)+0.13,...
                    [' ' num2str(nodes(i,1))],'Color','g','FontWeight','b','FontSize',8)
            end  
        end
    end

    
    % now plot the edges linking the nodes

    for i=1:num_edges
        N1=edges(i,2); 
        N2=edges(i,3); 
        plot([nodes(N1,2) nodes(N2,2)],[nodes(N1,3) nodes(N2,3)],'-k')
    end
    
    
    % plot entries without a real meaning. Useful just to create a legend
    % showing the colors used in the figure and their meaning
    legaux = zeros(3, 1);
    legaux(1) = plot(NaN,NaN,'-b');
    legaux(2) = plot(NaN,NaN,'-r');
    legaux(3) = plot(NaN,NaN,'-g');
    legend(legaux, 'Generic Node','Operation Node','Bay Node');
    
    title('Warehouse Map')
    xlabel(' [m] ');
    ylabel(' [m] ');

    grid on
    grid minor

    %% --- Total Times Vector construction ---
    
    fprintf(' \n \n BUILDING TOTAL TIMES VECTOR ... \n')
    
    % initialize total times vectors
    
    TNTV = zeros(num_nodes,1);

    TNOTV = zeros(num_nodes,1); 
    
    TNMTV = zeros(num_nodes,1);
    
    TBTV = zeros(num_bays,1);

    TBRTV = zeros(num_bays,1);
    
    TBOTV = zeros(num_bays,1);
    
    % initialize Operation Times Matrix
    
    Operation_Times_Matrix = zeros(num_operations,5);
    
    
    
    % the idea now is to scroll all the operations, to compute the times of each of
    % them, to update total times vector at the end of each iteration, for the purpose of 
    % understanding which percentage of time is spent on each node.
    % Partial Times matrices are replaced by partial time vectors, reset
    % every new operation, for memory saving purpose.
    % The path calculations are done automatically using Dijkstra
    % Algorithm, with or without intermediate nodes according to the value
    % of intermediate node flag

    iter_count = 0; %counts the number of runnings of Dijkstra algorithm. If equal to the number of operations --> OK. Automatic check is done at the end
    
    tt_time_start = tic; %for computation time
    
    for i=1:num_operations %scroll operations

        fprintf('\n \n')
        fprintf(' TOTAL TIMES VECTOR CALCULATION IS AT %.2f percent ... \n', round(100*i/num_operations,2))
        
        iter_count = iter_count + 1;

        partial_times_vector = zeros(num_nodes,1); %reset partial times vector for each iteration
        partial_bay_times_vector = zeros(num_bays,1); %reset partial bay times vector for each operation

        bay_node_name = round(operation_nodes_matrix(i,5),0); %get bay node for the operation in analysis


        SID = 0; %SID is the bay node for the operation in analysis

        % select the ID of the bay node, knowing its name
        for j=1:num_bays
            if bay_identities_matrix(j,2) == bay_node_name
                SID = bay_identities_matrix(j,1); 
            end
        end

        FID = operation_nodes_matrix(i,1); %FID is the operation node


        % NOW DECIDE IF USING STANDARD ROUTING OR EXTENDED ROUTING

        if (round(operation_nodes_matrix(i,8)) ~= 0) % IF Extended Routing

            % call function to calculate path through a sequence of intermediate nodes
            [dist,path] = dijkstra_intermediate_nodes(i,UPW_int_nodes_matrix,RET_int_nodes_matrix,SID,FID,nodes,edges);

            % update partial times vector
            for j = 1:size(path,2)
                partial_times_vector(path(j)) = partial_times_vector(path(j)) + crossing_time; %add crossing times to all the crossed nodes.
            end


        else % IF Standard Routing

            %call Dijkstra algorithm from bay to that node
            %dist is the distance from SID to FID, crossing the nodes indicated
            %in path vector
            [dist,path] = dijkstra_algorithm(nodes,edges,SID,FID); %call Dijkstra algorithm function to obtain minimal distance and related path


            %on the basis of the crossed nodes, add all the times
            for j = 1:size(path,2)
                partial_times_vector(path(j)) = partial_times_vector(path(j)) + 2 * crossing_time; %add crossing times to all the crossed nodes.
            end

        end 
        
        
        % before updating total times vector, produce the
        % Operation Times Matrix, for checking purposes
        
        % Operation Times Matrix : [ID  moving_time  op.time  bay_time  bay_recharge_time  total_operation_time] 
        Operation_Times_Matrix(i,1) = operation_nodes_matrix(i,1); % node ID
        Operation_Times_Matrix(i,2) = sum(partial_times_vector); %trip time (time in which the forklift is moving)
        Operation_Times_Matrix(i,3) = operation_nodes_matrix(i,2); %operation time
        Operation_Times_Matrix(i,4) = operation_nodes_matrix(i,6); %bay time
        Operation_Times_Matrix(i,5) = operation_nodes_matrix(i,6)*operation_nodes_matrix(i,7); %recharge bay time - time for which forklift can be charged by an eventual static WPT placed in the bay

        
        % update total times vector
        
        partial_times_vector(FID) = partial_times_vector(FID) + operation_nodes_matrix(i,2); %add operation times to the operation node
     
        partial_sum = sum(partial_times_vector) + operation_nodes_matrix(i,6); %sum all times related to the operation in this node, and add time spent in the bay

        Operation_Times_Matrix(i,6) = partial_sum; %report total operation time in operation times matrix, for data analysis purposes
        
        partial_times_vector = partial_times_vector / partial_sum; %weight each term of partial time vector by the partial sum

        partial_bay_times_vector(bay_node_name) = operation_nodes_matrix(i,6) / partial_sum; %weight the bay time by the partial sum

        partial_times_vector = partial_times_vector * operation_nodes_matrix(i,3); %weight for the probability to pick an object in that node
        partial_bay_times_vector(bay_node_name) = partial_bay_times_vector(bay_node_name) * operation_nodes_matrix(i,3); %weight for the probability to pick an object in that node

        TNTV = TNTV + partial_times_vector; %update total node times vector
        TBTV = TBTV + partial_bay_times_vector; %update total bay times vector

        TBRTV(bay_node_name) = TBRTV(bay_node_name) + partial_bay_times_vector(bay_node_name) * operation_nodes_matrix(i,7); %update total bay recharge times vector
        
        TNOTV(FID) = TNOTV(FID) + operation_nodes_matrix(i,2) * operation_nodes_matrix(i,3) / partial_sum; %update total node operation times vector
    end

    TBOTV = TBTV - TBRTV; %calculate total bay operation times vector
    
    TNMTV = TNTV - TNOTV; %calculate total node movement times vector
    

    
    time_check = sum(TNTV) + sum(TBTV); %IF 1 --> OK, checked later

    
    % define total times
    
    total_time = sum(TNTV); %perc. of time forklift is in the area of the modelled warehouse, both moving or operating;
    
    total_moving_time = sum(TNMTV); %perc. of time forklift is moving in the nodes of the warehouse;
    
    total_operation_time = sum(TNOTV); %perc. of time forklift is operating in the operation nodes of the warehouse;
    
    total_bay_time = sum(TBTV); %perc. of time forklift stays in all the bays;
    
    total_bay_recharge_time = sum(TBRTV); %perc. of time forklift stays in all the bays, while idle and so potentially charging;
    
    total_bay_operation_time = sum(TBOTV); %perc. of time forklift stays in all the bays, while operating there;
    
    clock_total_times_calculation = toc(tt_time_start); %computation time requested to calculate total times
    
    %% --- 1st Check Section
    
    
    % Time check
    
    if ((abs(time_check - 1) > 0.0001) || (abs(time_check - 1) < -0.0001))
        fprintf(' \n \n ERROR IN TIME CHECK DETECTED - TOTAL TIME IN TOTAL_TIMES_VECTOR IS NOT 1 OR ABOVE TOLERANCE \n')
        return
    end
    

    % Iter counts check
    
    if iter_count ~= num_operations
        fprintf(' \n \n ERROR IN ITER COUNT, NOT ENOUGH OR TOO MUCH OPERATIONS HAVE BEEN CONSIDERED \n ')
        return
    end
    
    
    % Total Times Vector check: TTV = TMTV + TOTV
    
    if TNTV ~= (TNMTV + TNOTV) 
        fprintf(' \n \n ERROR IN TOTAL TIMES VECTOR, TTV NOT EQUAL TO TMTV + TOTV \n ')
        return
    end
    
    
    % Total Bay Times Vector: TBTV = TBRTV + TBOTV
    
    if TBTV ~= (TBRTV + TBOTV) 
        fprintf(' \n \n ERROR IN TOTAL BAY TIMES VECTOR, TBTV NOT EQUAL TO TBRTV + TBOTV \n ')
        return
    end
    
    
    % Distance-2 Neighbors Check
    
    if ((size(Nodes_Neighbors_R2_Hor,1) ~= num_nodes) || (size(Nodes_Neighbors_R2_Hor,2) ~= 4))
        fprintf(' \n \n ERROR IN HOR NODES 2-NEIGHBORS - MATRIX SIZE NOT CORRECT \n ')
        return
    end
    
    if ((size(Nodes_Neighbors_R2_Ver,1) ~= num_nodes) || (size(Nodes_Neighbors_R2_Ver,2) ~= 4))
        fprintf(' \n \n ERROR IN VER NODES 2-NEIGHBORS - MATRIX SIZE NOT CORRECT \n ')
        return
    end
    
    
    % Radius-5 Neighbors Check
    
    if ((size(Nodes_Neighbors_D5_Hor,1) ~= num_nodes) || (size(Nodes_Neighbors_D5_Hor,2) ~= 2))
        fprintf(' \n \n ERROR IN HOR NODES R5-NEIGHBORS - MATRIX SIZE NOT CORRECT \n ')
        return
    end
    
    if ((size(Nodes_Neighbors_D5_Ver,1) ~= num_nodes) || (size(Nodes_Neighbors_D5_Ver,2) ~= 2))
        fprintf(' \n \n ERROR IN VER NODES R5-NEIGHBORS - MATRIX SIZE NOT CORRECT \n ')
        return
    end
     
    %% --- FIGURE 2 - Total Times Map construction ---

    % useful map to see the nodes in which the forklift spends more time
    % Label over each node indicates the total node time of it.
    
    total_times_vector_figure_2 = round ( TNTV * 100 , 3 );

    
    figure(2)
    hold on

    
    %plot the nodes
    plot(nodes(:,2),nodes(:,3),'b.')
    for i=1:num_nodes
        if nodes_excel_matrix(i,4) == 1
            text(nodes(i,2),nodes(i,3)+0.13,...
                [' ' num2str(total_times_vector_figure_2(i))],'Color','k','FontWeight','b','FontSize',8)
        else
            text(nodes(i,2),nodes(i,3)+0.13,...
                [' ' num2str(total_times_vector_figure_2(i))],'Color','b','FontWeight','b','FontSize',8)
        end
    end

    
    % now plot the edges linking the nodes
    for i=1:num_edges
        N1=edges(i,2);
        N2=edges(i,3);
        plot([nodes(N1,2) nodes(N2,2)],[nodes(N1,3) nodes(N2,3)],'-k')
    end

    grid on
    grid minor

    title(['Percentage total time spent on nodes. Percentage total time spent in bay is ' num2str(round(sum(TBTV)*100,2)) '%.'])

    
    % plot entries without a real meaning. Useful just to create a legend
    % showing the colors used in the figure and their meaning
    legaux = zeros(2, 1);
    legaux(1) = plot(NaN,NaN,'-b');
    legaux(2) = plot(NaN,NaN,'-k');
    legend(legaux, 'Generic Node','Operation Node')
    
    hold off

end

%
%
% END OF 1ST STEP
%
%

%% --- FIGURE 3, 4, 5 : Warehouse layout check ---


% Node category map: label over each node indicate its Node Category
figure(3)
hold on

% plot the nodes
plot(nodes(:,2),nodes(:,3),'b.')
for i=1:num_nodes
    text(nodes(i,2),nodes(i,3)+0.13,...
        [' ' num2str(nodes_categories(i))],'Color','b','FontWeight','b','FontSize',8)
end


% plot the edges linking the nodes
for i=1:num_edges
    N1=edges(i,2); 
    N2=edges(i,3); 
    plot([nodes(N1,2) nodes(N2,2)],[nodes(N1,3) nodes(N2,3)],'-k')
end

title('Nodes Category Map')
xlabel(' [m] ');
ylabel(' [m] ');

grid on
grid minor



% Horizontal Corridors map: label over each node indicate its Horizontal
% Corridor number. If 0, that node does not belong to anyone on them
figure(4)
hold on

% plot the nodes
plot(nodes(:,2),nodes(:,3),'b.')
for i=1:num_nodes
    text(nodes(i,2),nodes(i,3)+0.13,...
        [' ' num2str(round(nodes_excel_matrix(i,6),0))],'Color','b','FontWeight','b','FontSize',8)
end


% plot the segments linking the nodes
for i=1:num_edges
    N1=edges(i,2); 
    N2=edges(i,3); 
    plot([nodes(N1,2) nodes(N2,2)],[nodes(N1,3) nodes(N2,3)],'-k')
end

title('Nodes horizontal corridors map')
grid on
grid minor


% Vertical Corridors map: label over each node indicate its Vertical
% Corridor number. If 0, that node does not belong to anyone on them
figure(5)
hold on

plot(nodes(:,2),nodes(:,3),'b.')
for i=1:num_nodes
    text(nodes(i,2),nodes(i,3)+0.13,...
        [' ' num2str(round(nodes_excel_matrix(i,7),0))],'Color','b','FontWeight','b','FontSize',8)
end


% now plot the segments linking the nodes
for i=1:num_edges
    N1=edges(i,2); 
    N2=edges(i,3); 
    plot([nodes(N1,2) nodes(N2,2)],[nodes(N1,3) nodes(N2,3)],'-k') %plot all the segments 
end


title('Nodes vertical corridors map')
grid on
grid minor

%% --- Initial Energy Balance for Optimization ---


% Energy Lost

E_out_bay_op = P_cons_bay_op * ST_eff * total_bay_operation_time; %[J]

E_out_bay_rech = P_cons_bay_rech * ST_eff * total_bay_recharge_time; %[J]

E_out_node_mov = P_cons_node_mov * ST_eff * total_moving_time; %[J]

E_out_node_op = P_cons_node_op * ST_eff * total_operation_time; %[J]


% Total energy lost

E_out_turn = E_out_bay_op + E_out_bay_rech + E_out_node_mov + E_out_node_op; %[J]


% Energy intaked during breaks

E_in_breaks = Pn * static_efficiency * ST_breaks * k_breaks; % [J]


% Energetic requirement

Delta_E_turn_desired = Delta_SoC_desired * 0.01 * E_battery_max; % [J]

%% Optimization - Cost function and integer optimization variables

constraints_building_time_start = tic; %for computation time purposes


N_decision_var = 5*num_nodes + num_bays; % decision variables of the problem. 
% structure is [ x(i) , h(i) , v(i) , hc(i) , vc(i) , xb(i) ]


f = [ cost_dynamic_WPT*ones(1,num_nodes) zeros(1,4*num_nodes) cost_static_WPT*ones(1,num_bays)]; %objective function.
%   [            x                     h  v  hc  vc               xb                       ] 


x = zeros(N_decision_var,1); %initialization of decision variables, to check dimensions


% define integer decision variables (all)
for i = 1:N_decision_var
    intcon(i) = i;
end


equal_constr_row = 0; %counter of equality constraints
ineq_constr_row = 0; %counter of inequality constraints

%% Optimization - 1) Unicity constraint

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 1) UNICITY ... \n')

% See thesis for understanding the meaning of this constraint


for i=1:num_nodes %scroll nodes
    
    fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 1) UNICITY ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
    
    if nodes_categories(i) == 1 %if node is HORIZONTAL
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,i) = 3;
        Aeq(equal_constr_row,num_nodes+i) = -3;
        Aeq(equal_constr_row,3*num_nodes+i) = -3;
        Aeq(equal_constr_row,2*num_nodes+i) = -1;
        Aeq(equal_constr_row,4*num_nodes+i) = -1;

        beq(equal_constr_row) = 0;
        
        
    elseif nodes_categories(i) == 2 %if node is VERTICAL
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,i) = 3;
        Aeq(equal_constr_row,2*num_nodes+i) = -3;
        Aeq(equal_constr_row,4*num_nodes+i) = -3;
        Aeq(equal_constr_row,num_nodes+i) = -1;
        Aeq(equal_constr_row,3*num_nodes+i) = -1;
        
        beq(equal_constr_row) = 0;
        
        
    elseif nodes_categories(i) == 3 %if node is DOUBLE
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,i) = 1;
        Aeq(equal_constr_row,2*num_nodes+i) = -1;
        Aeq(equal_constr_row,4*num_nodes+i) = -1;
        Aeq(equal_constr_row,num_nodes+i) = -1;
        Aeq(equal_constr_row,3*num_nodes+i) = -1;
        
        beq(equal_constr_row) = 0;
        
    end
    
end

%% Optimization - 2) WPT placement impossibility constraint

% See thesis for understanding the meaning of this constraint

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 2) WPT PLACEMENT IMPOSSIBILITY ... \n')


for i=1:num_nodes %scroll nodes
    
    fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 2) WPT PLACEMENT IMPOSSIBILITY ... AT %.2f percent ... \n', round(100*i/num_nodes,2))

   
    if nodes_categories(i) == 4 %if node is IMPOSSIBLE
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,i) = 1;
        Aeq(equal_constr_row,num_nodes+i) = 1;
        Aeq(equal_constr_row,2*num_nodes+i) = 1;
        Aeq(equal_constr_row,3*num_nodes+i) = 1;
        Aeq(equal_constr_row,4*num_nodes+i) = 1;
        

        beq(equal_constr_row) = 0;
        
    end

end

%% Optimization - 3) Constraints on sum of HOR and VER nodes in corridors

% See thesis for understanding the meaning of this constraint


% 3.1 - For horizontal corridors only

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 3.1) SUM OF NODES IN HOR CORRIDORS ... \n')

for j=1:num_hor_corr %scroll HOR corridors
    
    fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 3.1) SUM OF NODES IN HOR CORRIDORS ... AT %.2f percent ... \n', round(100*j/num_hor_corr,2))

    
    equal_constr_row = equal_constr_row + 1; %define an equality constraint for each corridor
    Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
    beq(equal_constr_row) = 0;
    
    for i=1:num_nodes %scroll nodes
        
        if round(nodes_excel_matrix(i,6),0) == j %if that node belongs to that HOR corridor
            
            Aeq(equal_constr_row,num_nodes+i) = 1; %weight of h(i)
            Aeq(equal_constr_row,3*num_nodes+i) = -4; %weight of hc(i)
        end
        
    end
end


% 3.2 - For vertical corridors only

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 3.2) SUM OF NODES IN VER CORRIDORS ... \n')

for j=1:num_ver_corr %scroll VER corridors

    fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 3.2) SUM OF NODES IN VER CORRIDORS ... AT %.2f percent ... \n', round(100*j/num_ver_corr,2))
    
    equal_constr_row = equal_constr_row + 1; %define an equality constraint for each corridor
    Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
    beq(equal_constr_row) = 0;
    
    for i=1:num_nodes %scroll nodes
        
        if round(nodes_excel_matrix(i,7),0) == j %if that node belongs to that VER corridor
            
            Aeq(equal_constr_row,2*num_nodes+i) = 1; %weight of v(i)
            Aeq(equal_constr_row,4*num_nodes+i) = -4; %weight of vc(i)

        end
        
    end
end

%% Optimization - 4) Constraints on WPT Center position

% See thesis for understanding the meaning of this constraint


% 4.1) 

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 4.1) HOR WPT CENTER POSITION ... \n')

for i=1:num_nodes %scroll nodes
        
    if nnz(Nodes_Neighbors_R2_Hor(i,:)) == 4 %if true, it can be Hor Center
        
        fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 4.1) HOR WPT CENTER POSITION ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
        
        ineq_constr_row = ineq_constr_row + 1;
        A(ineq_constr_row,:) = zeros(1,N_decision_var);
    
        A(ineq_constr_row,3*num_nodes+i) = 4; %node that can be Hor Center 
        
        for iter=1:4 %scroll all its R2 Hor Neighbors
            
            j = Nodes_Neighbors_R2_Hor(i,iter);
            A(ineq_constr_row,num_nodes+j) = -1; %nodes that can be Hor Parts 
            
        end
    
        b(ineq_constr_row) = 0;
        
    end
    
end


% 4.2) 

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 4.2) VER WPT CENTER POSITION ... \n')

for i=1:num_nodes %scroll nodes
    
    if nnz(Nodes_Neighbors_R2_Ver(i,:)) == 4 %if true, it can be Ver Center
         
        fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 4.2) VER WPT CENTER POSITION ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
        
        ineq_constr_row = ineq_constr_row + 1;
        A(ineq_constr_row,:) = zeros(1,N_decision_var);
    
        A(ineq_constr_row,4*num_nodes+i) = 4; %node that can be Ver Center 
        
        for iter=1:4 %scroll all its R2 Ver Neighbors
            
            j = Nodes_Neighbors_R2_Ver(i,iter);
            A(ineq_constr_row,2*num_nodes+j) = -1; %nodes that can be Ver Parts 
            
        end
    
        b(ineq_constr_row) = 0;
        
    end
    
end

%% Optimization - 5) Constraints on WPT Part position

% See thesis for understanding the meaning of this constraint


% 5.1) 

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 5.1) HOR WPT PART POSITION ... \n')

for i=1:num_nodes %scroll nodes
    
    if (nodes_categories(i) == 1)||(nodes_categories(i) == 3) %if node is HORIZONTAL or CROSS
        
        fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 5.1) HOR WPT PART POSITION ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
        
        M = nnz(Nodes_Neighbors_R2_Hor(i,:)); %get all its R2 Hor neighbors.
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,num_nodes+i) = 1; %horizontal node under analysis 
        
        for iter=1:M %scroll all its R2 Hor neighbors.
            
            j = Nodes_Neighbors_R2_Hor(i,iter); %take ID of neighbor node
            Aeq(equal_constr_row,3*num_nodes+j) = -1; %nodes that can be Ver Centers 
            
        end
    
        beq(equal_constr_row) = 0;

    end
    
end


% 5.2) 

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 5.2) VER WPT PART POSITION ... \n')

for i=1:num_nodes %scroll nodes
    
    if (nodes_categories(i) == 2)||(nodes_categories(i) == 3) %if node is VERTICAL or DOUBLE
        
        fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 5.2) VER WPT PART POSITION ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
        
        M = nnz(Nodes_Neighbors_R2_Ver(i,:)); %get all its R2 Ver neighbors.
        
        equal_constr_row = equal_constr_row + 1;
        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        
        Aeq(equal_constr_row,2*num_nodes+i) = 1; %vertical node under analysis 
        
        for iter=1:M %scroll all its R2 Ver neighbors
            
            j = Nodes_Neighbors_R2_Ver(i,iter); %take ID of neighbor node
            Aeq(equal_constr_row,4*num_nodes+j) = -1; %nodes that can be Ver Centers 
            
        end
    
        beq(equal_constr_row) = 0;

    end
    
end

%% Optimization - 6) Constraints on Minimum Lenght of WPT

% See thesis for understanding the meaning of this constraint


fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 6) MIN LENGHT OF WPT ... \n')

for i=1:num_nodes %scroll nodes
    
    fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 6) MIN LENGHT OF WPT ... AT %.2f percent ... \n', round(100*i/num_nodes,2))
    
    % 6.1 - For Horizontal and Cross nodes
        
    if (nodes_categories(i) == 1) || (nodes_categories(i) == 3) %if node is Horizontal or Cross
        if nnz(Nodes_Neighbors_D5_Hor(i,:)) > 0 %if it has some D5 Hor Neighbors
                        
            ineq_constr_row = ineq_constr_row + 1;
            
            A(ineq_constr_row,:) = zeros(1,N_decision_var);
            
            A(ineq_constr_row,3*num_nodes+i) = 1; %node i
            
            for j = 1:2 
                if Nodes_Neighbors_D5_Hor(i,j) ~= 0
                    
                    A(ineq_constr_row,3*num_nodes+Nodes_Neighbors_D5_Hor(i,j)) = -1; %D5 Hor neighbors
                    
                end
            end
            
            b(ineq_constr_row) = 0;
            
        else %if it does not have D5 Hor Neighbors
            
            % state that it cannot be WPT center
            
            equal_constr_row = equal_constr_row + 1;
            
            A(equal_constr_row,:) = zeros(1,N_decision_var);
            A(equal_constr_row,3*num_nodes+i) = 1; %node i
            
            b(equal_constr_row) = 0;
            
        end
        
        
    end
    
    
    % 6.2 - For Vertical and Cross nodes
       
    if (nodes_categories(i) == 2) || (nodes_categories(i) == 3) %if node is Vertical or Cross
        if nnz(Nodes_Neighbors_D5_Ver(i,:)) > 0 %if it has some D5 Ver Neighbors
                        
            ineq_constr_row = ineq_constr_row + 1;
            
            A(ineq_constr_row,:) = zeros(1,N_decision_var);
            
            A(ineq_constr_row,4*num_nodes+i) = 1; %node i
            
            for j = 1:2 
                if Nodes_Neighbors_D5_Ver(i,j) ~= 0
                    
                    A(ineq_constr_row,4*num_nodes+Nodes_Neighbors_D5_Ver(i,j)) = -1; %D5 Ver Neighbors
                    
                end
            end
            
            b(ineq_constr_row) = 0;
            
        else %if it does not have D5 Ver Neighbors
            
            % state that it cannot be WPT center
            
            equal_constr_row = equal_constr_row + 1;
            
            A(equal_constr_row,:) = zeros(1,N_decision_var);
            A(equal_constr_row,4*num_nodes+i) = 1; %node i
            
            b(equal_constr_row) = 0;
            
        end
    end
    
end

%% Optimization - 7) Constraints on bay WPT impossibility placement


% See thesis for understanding the meaning of this constraint

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 7) IMPOSSIBLE BAY WPT ... \n')


for i = 1:num_bays %scroll bays
    
    if round(bay_identities_matrix(i,3),0) == 0 %if bay node i cannot host a WPT
        
        equal_constr_row = equal_constr_row + 1;

        Aeq(equal_constr_row,:) = zeros(1,N_decision_var);
        Aeq(equal_constr_row,5*num_nodes+i) = 1;

        beq(equal_constr_row) = 0;
        
    end
    
end

%% Optimization - 8) SoC Constraint

% See thesis for understanding the meaning of this constraint

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - 8) SOC ... \n')


% E_in_dynamic

ineq_constr_row = ineq_constr_row + 1;

A(ineq_constr_row,:) = zeros(1,N_decision_var);

for i = 1:num_nodes
    A(ineq_constr_row,i) = -1 * TNTV(i) * ST_eff * Pn * dynamic_efficiency;
end


% E_in_bays

iter = 0;

for i = 5*num_nodes+1:5*num_nodes+num_bays
    iter = iter + 1;
    A(ineq_constr_row,i) = -1 * TBRTV(iter) * ST_eff * Pn * static_efficiency;
end


b(ineq_constr_row) = -Delta_E_turn_desired + E_in_breaks - E_out_turn;

%% Optimization - Upper and Lower bounds

fprintf(' \n \n BUILDING CONSTRAINTS FOR LP OPTIMIZATION - BOUNDS ... \n')

% upper and lower bounds on integer decision variables
% They can be either 0 or 1 (binary optimization variables)

ub=ones(numel(x),1);
lb=zeros(numel(x),1);


% save matrices in sparse form to save memory.

A = sparse(A);
b = sparse(b);
Aeq = sparse(Aeq);
beq = sparse(beq);
f = sparse(f);
ub = sparse(ub);
lb = sparse(lb);
intcon = sparse(intcon);
x = sparse(x);

clock_constraints_building = toc(constraints_building_time_start); %duration of the constraints building phase

%% Optimization - Options and MILP FORMULATION

fprintf(' \n \n LINEAR PROGRAMMING OPTIMIZATION IN PROCESS ... \n\n\n')

time_start = tic; %for computation time purposes

if FLAG_SOLVER_STOP == 1
    options = optimoptions('intlinprog','CutMaxIterations',25,'CutGeneration','intermediate','IntegerPreprocess','advanced','HeuristicsMaxNodes',100,'Heuristics','advanced','MaxFeasiblePoints',1);
else
    options = optimoptions('intlinprog','CutMaxIterations',25,'CutGeneration','intermediate','IntegerPreprocess','advanced','HeuristicsMaxNodes',100,'Heuristics','advanced','MaxTime',28800);
end


[x,fval,exitflag,output] = intlinprog(f,intcon,A,b,Aeq,beq,lb,ub,options);

x = round(x, 0); %necessary to exactly round all 0s and 1s to perfect integers

clock_optimization_elapsed_time = toc(time_start); %time needed for the MILP optimization

%% --- results of optimization ---

fprintf(' \n \n OPTIMIZATION ENDED ... \n')


WPT_COST = fval % paste on CW total (optimal) cost of the system.


% Build a recapping matrix for nodes, indicating if a WPT is present of
% them and eventually its type, and the total time a forklift statistically
% spends on that node

% NODES_INFO STRUCTURE : [ID,  X,  Y,  WPT(y/n), Tot_time_perc, Node type] 

% Node type: 0 = NO WPT
% Node type: 1 = Horizontal WPT
% Node type: 2 = Vertical WPT
% Node type: 11 = Horizontal Center WPT
% Node type: 22 = Vertical Center WPT

NODES_INFO(:,[1:3]) = nodes(:,[1:3]); % same infos contained in nodes matrix.
NODES_INFO(:,4) = x(1:num_nodes); %fourth column of nodes info is the presence or not of the WPT
NODES_INFO(:,5) = TNTV*100; %fifth column of nodes_info is the percentage total times vector

WPT_STATUS = [x(1:num_nodes) x(num_nodes+1:2*num_nodes) x(2*num_nodes+1:3*num_nodes) x(3*num_nodes+1:4*num_nodes) x(4*num_nodes+1:5*num_nodes)];

% WPT STATUS matrix indicates, for each node, if a WPT is present on it, and
% indicates which type of WPT it is. This matrix will be used for building
% the WPT placement figure.

% WPT_STATUS = [ WPT,  HOR,  VER,  HOR_CEN,  VER_CEN ]
% 1 indicates a WPT of that type is present on the node. 0 Otherwise

for i=1:num_nodes
    if WPT_STATUS(i,1) == 1 %if there is a wpt
        
        j = find(WPT_STATUS(i,2:5) == 1);
        
        switch j
            case 1 % HOR WPT
                NODES_INFO(i,6) = 1;
            case 2 % VER WPT
                NODES_INFO(i,6) = 2;
            case 3 % HOR WPT CENTER
                NODES_INFO(i,6) = 11;
            case 4 % VER WPT CENTER
                NODES_INFO(i,6) = 22;
        end
        
    else
        NODES_INFO(i,6) = 0;
    end
end

% Build an output matrix used as a list of the locations of WPT Centers,
% alongside the parts in those WPTs. It's called WPT_SYSTEM_OUTPUT.
% WPT_SYSTEM_OUTPUT = [Center ID;  TYPE(11=HOR,22=VER) 0  N1  N2  N3  N4]

iter = 0;

WPT_SYSTEM_OUTPUT = zeros(num_nodes,7);

for i = 1:num_nodes %scroll nodes
    
    if NODES_INFO(i,6) == 11 %if node i is a HOR CENTER
        
        iter = iter + 1;
        
        WPT_SYSTEM_OUTPUT(iter,1) = i; %center ID
        WPT_SYSTEM_OUTPUT(iter,2) = 11; %type
        WPT_SYSTEM_OUTPUT(iter,3) = 0; %for separating
        
        nn = Nodes_Neighbors_R2_Hor(i,:); %take Radius-2 Hor Neighbors of the center (they have a Part)
        nn = sort(nn); %sort them in ascending order
        
        WPT_SYSTEM_OUTPUT(iter,4:7) = nn; %post WPT parts on the matrix

    elseif NODES_INFO(i,6) == 22 %if node i is a VER CENTER
        
        iter = iter + 1;
        
        WPT_SYSTEM_OUTPUT(iter,1) = i; %center ID
        WPT_SYSTEM_OUTPUT(iter,2) = 22; %type
        WPT_SYSTEM_OUTPUT(iter,3) = 0; %for separating
        
        nn = Nodes_Neighbors_R2_Ver(i,:); %take Radius-2 Ver Neighbors of the center (they have a Part)
        nn = sort(nn); %sort them in ascending order
        
        WPT_SYSTEM_OUTPUT(iter,4:7) = nn; %post WPT parts on the matrix
        
    end
    
end


% Build a recapping matrix for the bays, indicating if a WPT is to be placed on them 

BAYS_INFO = zeros(num_bays,3); %[ bay_node_ID  bay_name  WPT_in_bay_Y/N ]

BAYS_INFO(:,1) = bay_identities_matrix(:,1);
BAYS_INFO(:,2) = bay_identities_matrix(:,2);
BAYS_INFO(:,3) = x(5*num_nodes+1:5*num_nodes+num_bays); 

%% --- FIGURE 6 - Map of the WPT positions ---

% Warehouse map with WPT parts position highlighted.
% Red-circled nodes are the ones with a WPT placed.


figure(6) 

hold on

%plot the nodes
plot(NODES_INFO(:,2),NODES_INFO(:,3),'b.')
for i=1:num_nodes
    text(NODES_INFO(i,2),NODES_INFO(i,3)+0.13,...
        [' ' num2str(NODES_INFO(i,1))],'Color','b','FontWeight','b')
end

% plot the segments linking the nodes
for i=1:num_edges
    N1=edges(i,2);
    N2=edges(i,3);
    plot([NODES_INFO(N1,2) NODES_INFO(N2,2)],[NODES_INFO(N1,3) NODES_INFO(N2,3)],'-k') 
end


grid on
grid minor

%if WPT is present, plot in on the map in red
for i=1:num_nodes
    if NODES_INFO(i,4) == 1
                   
    plot(nodes(i,2),nodes(i,3),'sr-','lineWidth',2)

    text(nodes(i,2),nodes(i,3)+0.13,...
        [' ' num2str(nodes(i,1))],'Color','r','FontWeight','b') %add node number as a label. N.B. Fontweight 'b' means BOLD font
     
    end
end


title(['Optimal WPT positions. Total cost is ' num2str(WPT_COST) ' . '])

% plot entries without a real meaning. Useful just to create a legend
% showing the colors used in the figure and their meaning
legaux = zeros(2,1);
legaux(1) = plot(NaN,NaN,'.b');
legaux(2) = plot(NaN,NaN,'sr');
legend(legaux, 'Generic Node','Node with a WPT part');

hold off

%% --- FIGURE 7 - Map of the Lighted segments ---

% figure 7 is the warehouse map with the segment continuity highlighted. 
% Red segments are the ones with a WPT part placed in both their nodes.
% Useful for conceptualization purposes, and to see the macro-areas of WPT
% installation.

figure(7)

hold on

%plot the nodes
plot(NODES_INFO(:,2),NODES_INFO(:,3),'k.')
for i=1:num_nodes
    text(NODES_INFO(i,2),NODES_INFO(i,3)+0.17,...
        [' ' num2str(NODES_INFO(i,1))],'Color','b','FontWeight','b','LineWidth',1)
end

% plot the segments linking the nodes
for i=1:num_edges
    N1=edges(i,2); 
    N2=edges(i,3);
    if (x(N1) == 1 & x(N2) == 1) %if segment has to be lighted
        plot([NODES_INFO(N1,2) NODES_INFO(N2,2)],[NODES_INFO(N1,3) NODES_INFO(N2,3)],'-r','LineWidth',3)
    else
        plot([NODES_INFO(N1,2) NODES_INFO(N2,2)],[NODES_INFO(N1,3) NODES_INFO(N2,3)],'-k','LineWidth',3)
    end
end


grid on
grid minor

%if WPT is present, plot it on the map in red
for i=1:num_nodes
    if NODES_INFO(i,4) == 1 
                   
       text(nodes(i,2),nodes(i,3)+0.17,...
        [' ' num2str(nodes(i,1))],'Color','r','FontWeight','b','LineWidth',1)
            
    end
end

title(['Optimal WPT positions. Total cost is ' num2str(WPT_COST) ' . ' ])

% plot entries without a real meaning. Useful just to create a legend
% showing the colors used in the figure and their meaning
legaux = zeros(2,1);
legaux(1) = plot(NaN,NaN,'-k','lineWidth',2);
legaux(2) = plot(NaN,NaN,'-r','lineWidth',2);
legend(legaux, 'Edges without WPT','Edges with WPT');

hold off

%% --- FIGURE 8 - Map of the TOTAL TIME PERCENTAGE OF THE NODES ---

% useful map to see the nodes in which the forklift spends more time, in
% relation to the WPT positions. Red nodes are the ones with a WPT placed.
% Label over a node indicate its percentage total time.

total_times_vector_figure = NODES_INFO(:,5);

total_times_vector_figure = round ( total_times_vector_figure , 3 );

figure(8) 
hold on

%plot the nodes

plot(NODES_INFO(:,2),NODES_INFO(:,3),'b.') % plot the nodes
for i=1:num_nodes
    text(NODES_INFO(i,2),NODES_INFO(i,3)+0.13,...
        [' ' num2str(total_times_vector_figure(i))],'Color','b','FontWeight','b','FontSize',8)
end

% now plot the segments linking the nodes

for i=1:num_edges
    N1=edges(i,2);
    N2=edges(i,3);
    plot([NODES_INFO(N1,2) NODES_INFO(N2,2)],[NODES_INFO(N1,3) NODES_INFO(N2,3)],'-k')
end


grid on
grid minor


%if WPT is present, plot in on the map in red
for i=1:num_nodes
    if NODES_INFO(i,4) == 1
                   
    plot(NODES_INFO(i,2),NODES_INFO(i,3),'r.','lineWidth',2)

    text(NODES_INFO(i,2),NODES_INFO(i,3)+0.13,...
        [' ' num2str(total_times_vector_figure(i))],'Color','r','FontWeight','b','FontSize',8)
             
    end
end

title('Percentage total time spent on nodes')

% plot entries without a real meaning. Useful just to create a legend
% showing the colors used in the figure and their meaning
legaux = zeros(2,1);
legaux(1) = plot(NaN,NaN,'ob','lineWidth',2);
legaux(2) = plot(NaN,NaN,'or','lineWidth',2);
legend(legaux, 'Nodes without a WPT','Nodes with WPT part');

hold off

%% --- FIGURE 9 - Map of the REAL WPT ---

% Fundamental Output Map. The different WPTs are indicated as separate black segments.
% WPT center is indicated by a black circle.

figure(9)

hold on
plot(NODES_INFO(:,2),NODES_INFO(:,3),'k.')


for i=1:num_edges
    N1=edges(i,2);
    N2=edges(i,3);
    plot([NODES_INFO(N1,2) NODES_INFO(N2,2)],[NODES_INFO(N1,3) NODES_INFO(N2,3)],'-b','LineWidth',1)
end

grid on
grid minor

for i=1:num_nodes
    
    %search for horizontal centers
    if round(NODES_INFO(i,6),0) == 11 %if node i is an horizontal center
        
        for iter = 1:4
            j = Nodes_Neighbors_R2_Hor(i,iter); %take R2 Hor neighbors ID
            plot_matrix(iter,:) = [ Nodes_Neighbors_R2_Hor(i,iter) NODES_INFO(j,2)]; %build plot matrix
            % plot matrix structure : [ NEIGHBOR   X ]
            
            %color the node in red
            plot(nodes(j,2),nodes(j,3),'sr','lineWidth',0.5)

        end
        %once built plot matrix, search for x max and x min of the hor real WPT
        plot_max_coord = max(plot_matrix(:,2));
        plot_min_coord = min(plot_matrix(:,2));
        plot_fixed_coord = NODES_INFO(i,3); %y is fixed for hor WPT
        plot([plot_min_coord plot_max_coord],[plot_fixed_coord plot_fixed_coord],'-k','LineWidth',3) %plot the WPT
        
        %color the node center in black
        plot(nodes(i,2),nodes(i,3),'sk-','lineWidth',1.5)

        %color node center ID in black
        text(nodes(i,2),nodes(i,3)+0.17,...
            [' ' num2str(nodes(i,1))],'Color','b','FontWeight','b','LineWidth',1)
        
    end
    
    %search for vertical centers
    if round(NODES_INFO(i,6),0) == 22 %if node i is a vertical center
        
        for iter = 1:4
            j = Nodes_Neighbors_R2_Ver(i,iter); %take R2 Ver neighbors ID
            plot_matrix(iter,:) = [ Nodes_Neighbors_R2_Ver(i,iter) NODES_INFO(j,3)]; %build plot matrix
            % plot matrix structure : [ NEIGHBOR   Y ]
            
            %color the node in red
            plot(nodes(j,2),nodes(j,3),'sr','lineWidth',0.5)

        end
        %once built plot matrix, search for y max and y min of the ver real WPT
        plot_max_coord = max(plot_matrix(:,2));
        plot_min_coord = min(plot_matrix(:,2));
        plot_fixed_coord = NODES_INFO(i,2); %x is fixed for ver WPT
        plot([plot_fixed_coord plot_fixed_coord],[plot_min_coord plot_max_coord],'-k','LineWidth',3) %plot the WPT
        
        %color the node center in black
        plot(nodes(i,2),nodes(i,3),'sk-','lineWidth',1.5)

        %color node center ID in black
        text(nodes(i,2),nodes(i,3)+0.17,...
            [' ' num2str(nodes(i,1))],'Color','b','FontWeight','b','LineWidth',1)
        
    end
    
    
end

% if a bay has a static WPT on it, plot its bay node in red

for i = 1:num_bays
    
    bay_node = BAYS_INFO(i,1);
    
    
    if BAYS_INFO(i,3) == 1
        
        plot(NODES_INFO(bay_node,2),NODES_INFO(bay_node,3),'or','lineWidth',2) % plot the bay node as a red circle
        
    end

end


title(['Real WPT Locations. Total cost is ' num2str(WPT_COST) ' . '])
xlabel(' [m] ');
ylabel(' [m] ');

% plot entries without a real meaning. Useful just to create a legend
% showing the colors used in the figure and their meaning
legaux = zeros(4,1);
legaux(1) = plot(NaN,NaN,'-k','lineWidth',2);
legaux(2) = plot(NaN,NaN,'sk','lineWidth',2);
legaux(3) = plot(NaN,NaN,'sr','lineWidth',2);
legaux(4) = plot(NaN,NaN,'or','lineWidth',2);
legend(legaux, 'WPT','WPT center','WPT part','Bay static WPT');

%% --- Energetic check, to verify if SOC constraint is respected ---

E_in_dynamic = 0; % [J]

E_in_bays = 0; % [J]

for i=1:num_nodes
        
    E_in_dynamic = E_in_dynamic + Pn * dynamic_efficiency * ST_eff * NODES_INFO(i,5) / 100 * NODES_INFO(i,4);
        
end

for i = 1:num_bays
    
    E_in_bays = E_in_bays + Pn * static_efficiency * ST_eff * TBRTV(i) * BAYS_INFO(i,3);

end


Delta_E_turn = E_in_dynamic + E_in_bays + E_in_breaks - E_out_turn  % [J]

clock_optimization_elapsed_time %computational time requested by MILP optimization step [s]

clock_elapsed_time_overall = toc(time_start_overall) %computational time requested by the overall code [s]

Delta_SOC_turn_perc = Delta_E_turn / E_battery_max *100 %delta_SOC percentage at the end of the turn

